Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports P2PSample.PortSIP



Partial Public Class Form1
	Inherits Form
	Implements SIPCallbackEvents
	Public Const MAX_LINES As Integer = 9
	' Maximum lines
	Public Const LINE_BASE As Integer = 1

	Private _CallSessions As Session() = New Session(MAX_LINES - 1) {}

	Private _SIPInited As Boolean = False
	Private _CurrentlyLine As Integer = LINE_BASE

	Private _sdkLib As PortSIPLib


	'''///////////////////////////////////////////////////////////////////////
	'''///////////////////////////////////////////////////////////////////////

	Private Function findSession(sessionId As Integer) As Integer
		Dim index As Integer = -1
		For i As Integer = LINE_BASE To MAX_LINES - 1
			If _CallSessions(i).getSessionId() = sessionId Then
				index = i
				Exit For
			End If
		Next

		Return index
	End Function

	Private Function GetBytes(str As String) As Byte()
		Return System.Text.Encoding.[Default].GetBytes(str)
	End Function

	Private Function GetString(bytes As Byte()) As String
		Return System.Text.Encoding.[Default].GetString(bytes)
	End Function

	Private Function getLocalIP() As String
		Dim localIP As New StringBuilder()
		localIP.Length = 64
		Dim nics As Integer = _sdkLib.getNICNums()
		For i As Integer = 0 To nics - 1
			_sdkLib.getLocalIpAddress(i, localIP, 64)
			If localIP.ToString().IndexOf(":") = -1 Then
				' No ":" in the IP then it's the IPv4 address, we use it in our sample
				Exit For
				' the ":" is occurs in the IP then this is the IPv6 address.
				' In our sample we don't use the IPv6.
			Else

			End If
		Next

		Return localIP.ToString()
	End Function

	Private Sub updatePrackSetting()
		If Not _SIPInited Then
			Return
		End If

		If checkBoxPRACK.Checked Then
			_sdkLib.setReliableProvisional(2)
		Else
			_sdkLib.setReliableProvisional(0)
		End If
	End Sub

	Function joinConference(ByVal index As Int32) As Integer
		If _SIPInited = False Then
			Return 0
		End If
		If CheckBoxConf.Checked = False Then
			Return 0
		End If

		_sdkLib.setRemoteVideoWindow(_CallSessions(_CurrentlyLine).getSessionId(), IntPtr.Zero)
		_sdkLib.joinToConference(_CallSessions(index).getSessionId())

		' We need to unhold the line in conference
		If _CallSessions(index).getSessionState() Then
			_sdkLib.unHold(_CallSessions(index).getSessionId())
			_CallSessions(index).setHoldState(False)
		End If

		Return 0
	End Function

	Private Sub loadDevices()
		If _SIPInited = False Then
			Return
		End If

		Dim num As Integer = _sdkLib.getNumOfPlayoutDevices()
		For i As Integer = 0 To num - 1
			Dim deviceName As New StringBuilder()
			deviceName.Length = 256

			If _sdkLib.getPlayoutDeviceName(i, deviceName, 256) = 0 Then
				ComboBoxSpeakers.Items.Add(deviceName.ToString())
			End If

			ComboBoxSpeakers.SelectedIndex = 0
		Next


		num = _sdkLib.getNumOfRecordingDevices()
		For i As Integer = 0 To num - 1
			Dim deviceName As New StringBuilder()
			deviceName.Length = 256

			If _sdkLib.getRecordingDeviceName(i, deviceName, 256) = 0 Then
				ComboBoxMicrophones.Items.Add(deviceName.ToString())
			End If

			ComboBoxMicrophones.SelectedIndex = 0
		Next


		num = _sdkLib.getNumOfVideoCaptureDevices()
		For i As Integer = 0 To num - 1
			Dim uniqueId As New StringBuilder()
			uniqueId.Length = 256
			Dim deviceName As New StringBuilder()
			deviceName.Length = 256

			If _sdkLib.getVideoCaptureDeviceName(i, uniqueId, 256, deviceName, 256) = 0 Then
				ComboBoxCameras.Items.Add(deviceName.ToString())
			End If

			ComboBoxCameras.SelectedIndex = 0
		Next


		Dim volume As Integer = _sdkLib.getSpeakerVolume()
		TrackBarSpeaker.SetRange(0, 255)
		TrackBarSpeaker.Value = volume

		volume = _sdkLib.getMicVolume()
		TrackBarMicrophone.SetRange(0, 255)
		TrackBarMicrophone.Value = volume

	End Sub



	Private Sub InitSettings()
		If _SIPInited = False Then
			Return
		End If
		_sdkLib.enableAEC(checkBoxAEC.Checked)
		_sdkLib.enableAEC(checkBoxVAD.Checked)
		_sdkLib.enableCNG(checkBoxCNG.Checked)
		_sdkLib.enableAGC(checkBoxAGC.Checked)
		_sdkLib.enableANS(checkBoxANS.Checked)

		_sdkLib.setDoNotDisturb(CheckBoxDND.Checked)
	End Sub


	Private Sub SetSRTPType()
		If _SIPInited = False Then
			Return
		End If

		Dim SRTPPolicy As SRTP_POLICY = SRTP_POLICY.SRTP_POLICY_NONE

		Select Case ComboBoxSRTP.SelectedIndex
			Case 0
				SRTPPolicy = SRTP_POLICY.SRTP_POLICY_NONE
				Exit Select

			Case 1
				SRTPPolicy = SRTP_POLICY.SRTP_POLICY_PREFER
				Exit Select

			Case 2
				SRTPPolicy = SRTP_POLICY.SRTP_POLICY_FORCE
				Exit Select
		End Select

		_sdkLib.setSrtpPolicy(SRTPPolicy, False)
	End Sub


	Private Sub SetVideoResolution()
		If _SIPInited = False Then
			Return
		End If

		Dim width As Int32 = 352
		Dim height As Int32 = 288

		Select Case ComboBoxVideoResolution.SelectedIndex
			Case 0
				width = 176
				height = 144
				Exit Select
			Case 1
				width = 352
				height = 288
				Exit Select
			Case 2
				width = 640
				height = 480
				Exit Select
			Case 3
				width = 800
				height = 600
				Exit Select
			Case 4
				width = 1024
				height = 768
				Exit Select
			Case 5
				width = 1280
				height = 720
				Exit Select
			Case 6
				width = 320
				height = 240
				Exit Select
		End Select

		_sdkLib.setVideoResolution(width, height)
	End Sub


	Private Sub SetVideoQuality()
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setVideoBitrate(_CallSessions(_CurrentlyLine).getSessionId(), TrackBarVideoQuality.Value)
	End Sub


	' Default we just using PCMU, PCMA, and G.279
	Private Sub InitDefaultAudioCodecs()
		If _SIPInited = False Then
			Return
		End If


		_sdkLib.clearAudioCodec()


		' Default we just using PCMU, PCMA, G729
		_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_PCMU)
		_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_PCMA)
		_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_G729)


		_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_DTMF)
		' for DTMF as RTP Event - RFC2833
	End Sub




	Private Sub UpdateAudioCodecs()
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.clearAudioCodec()

		If checkBoxPCMU.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_PCMU)
		End If


		If checkBoxPCMA.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_PCMA)
		End If


		If checkBoxG729.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_G729)
		End If


		If checkBoxILBC.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_ILBC)
		End If


		If checkBoxGSM.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_GSM)
		End If


		If checkBoxAMR.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_AMR)
		End If

		If CheckBoxG722.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_G722)
		End If

		If CheckBoxSpeex.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_SPEEX)
		End If

		If checkBoxAMR.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_AMRWB)
		End If

		If CheckBoxAMRwb.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_AMRWB)
		End If

		If CheckBoxSpeexWB.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_SPEEXWB)
		End If

		If CheckBoxG7221.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_G7221)
		End If

		If checkBoxOpus.Checked = True Then
			_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_OPUS)
		End If


		_sdkLib.addAudioCodec(AUDIOCODEC_TYPE.AUDIOCODEC_DTMF)

	End Sub


	Private Sub UpdateVideoCodecs()
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.clearVideoCodec()

		If checkBoxH263.Checked = True Then
			_sdkLib.addVideoCodec(VIDEOCODEC_TYPE.VIDEO_CODEC_H263)
		End If

		If checkBoxH2631998.Checked = True Then
			_sdkLib.addVideoCodec(VIDEOCODEC_TYPE.VIDEO_CODEC_H263_1998)
		End If

		If checkBoxH264.Checked = True Then
			_sdkLib.addVideoCodec(VIDEOCODEC_TYPE.VIDEO_CODEC_H264)
		End If

		If checkBoxVP8.Checked = True Then
			_sdkLib.addVideoCodec(VIDEOCODEC_TYPE.VIDEO_CODEC_VP8)
		End If
	End Sub



	Private Sub deRegisterFromServer()
		If _SIPInited = False Then
			Return
		End If

		For i As Integer = LINE_BASE To MAX_LINES - 1
			If _CallSessions(i).getRecvCallState() = True Then
				_sdkLib.rejectCall(_CallSessions(i).getSessionId(), 486)
			ElseIf _CallSessions(i).getSessionState() = True Then
				_sdkLib.hangUp(_CallSessions(i).getSessionId())
			End If

			_CallSessions(i).reset()
		Next


		If _SIPInited = True Then
			_sdkLib.removeUser()
			_sdkLib.unInitialize()

			'
			' MUST called after _sdkLib.unInitliaze();
			'
			_sdkLib.releaseCallbackHandlers()
			_SIPInited = False
		End If


		ListBoxSIPLog.Items.Clear()

		ComboBoxLines.SelectedIndex = 0
		_CurrentlyLine = LINE_BASE

		ComboBoxSpeakers.Items.Clear()
		ComboBoxMicrophones.Items.Clear()
		ComboBoxCameras.Items.Clear()

	End Sub

	'''////////////////////////////////////////////////////////////////////////////////////////////////////////
	'''////////////////////////////////////////////////////////////////////////////////////////////////////////


	Public Sub New()
		InitializeComponent()
	End Sub

	Private Sub Form1_Load(sender As Object, e As EventArgs)

		' Create the call sessions array, allows maximum 500 lines,
		' but we just use 8 lines with this sample, we need a class to save the call sessions information

		Dim i As Integer = 0
		For i = 0 To MAX_LINES - 1
			_CallSessions(i) = New Session()
			_CallSessions(i).reset()
		Next

		_SIPInited = False
		_CurrentlyLine = LINE_BASE


		TrackBarSpeaker.SetRange(0, 255)
		TrackBarSpeaker.Value = 0

		TrackBarMicrophone.SetRange(0, 255)
		TrackBarMicrophone.Value = 0


		ComboBoxSRTP.Items.Add("None")
		ComboBoxSRTP.Items.Add("Prefer")
		ComboBoxSRTP.Items.Add("Force")

		ComboBoxSRTP.SelectedIndex = 0



		ComboBoxVideoResolution.Items.Add("QCIF")
		ComboBoxVideoResolution.Items.Add("CIF")
		ComboBoxVideoResolution.Items.Add("VGA")
		ComboBoxVideoResolution.Items.Add("SVGA")
		ComboBoxVideoResolution.Items.Add("XVGA")
		ComboBoxVideoResolution.Items.Add("720P")
		ComboBoxVideoResolution.Items.Add("QVGA")

		ComboBoxVideoResolution.SelectedIndex = 1



		ComboBoxLines.Items.Add("Line-1")
		ComboBoxLines.Items.Add("Line-2")
		ComboBoxLines.Items.Add("Line-3")
		ComboBoxLines.Items.Add("Line-4")
		ComboBoxLines.Items.Add("Line-5")
		ComboBoxLines.Items.Add("Line-6")
		ComboBoxLines.Items.Add("Line-7")
		ComboBoxLines.Items.Add("Line-8")


		ComboBoxLines.SelectedIndex = 0
	End Sub

	Private Sub Form1_FormClosing(sender As Object, e As FormClosingEventArgs)
		If _SIPInited = False Then
			Return
		End If

		deRegisterFromServer()
	End Sub

	Private Sub Button2_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		deRegisterFromServer()
	End Sub

	Private Sub Button1_Click(sender As Object, e As EventArgs)
		If _SIPInited = True Then
			MessageBox.Show("You are already logged in.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If


		If TextBoxUserName.Text.Length <= 0 Then
			MessageBox.Show("The user name does not allows empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If


		If TextBoxPassword.Text.Length <= 0 Then
			MessageBox.Show("The password does not allows empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim userName As String = TextBoxUserName.Text
		Dim password As String = TextBoxPassword.Text
		If textBoxLocalPort.Text.Length <= 0 Then
			MessageBox.Show("The local SIP port does not allows empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim localSipPort As Integer = Integer.Parse(textBoxLocalPort.Text)
		If localSipPort <= 0 OrElse localSipPort > 65535 Then
			MessageBox.Show("The local SIP port out of range.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If


		'
		' Create the class instance of PortSIP VoIP SDK, you can create more than one instances and 
		' each instance register to a SIP server to support multiples accounts & providers.
		' for example:
		'
		'            _sdkLib1 = new PortSIPLib(1, 1, this);
		'            _sdkLib2 = new PortSIPLib(2, 2, this);
		'            _sdkLib3 = new PortSIPLib(3, 3, this);
		'            



		_sdkLib = New PortSIPLib(Me)

		'
		' Create and set the SIP callback handers, this MUST called before
		' _sdkLib.initialize();
		'
		_sdkLib.createCallbackHandlers()


		Dim logFilePath As String = "d:\"
		' The log file path, you can change it - the folder MUST exists
		Dim agent As String = "PortSIP VoIP SDK"
		Dim errorCode As Integer = 0


		Dim stunServer As String = [String].Empty
		Dim transport As TRANSPORT_TYPE = TRANSPORT_TYPE.TRANSPORT_UDP

		Dim ip As String = getLocalIP()

		' Initialize the SDK
		Dim rt As Int32 = _sdkLib.initialize(transport, ip, localSipPort, PORTSIP_LOG_LEVEL.PORTSIP_LOG_NONE, logFilePath, MAX_LINES, agent, 0, 0, "/", "", False)


		If rt <> 0 Then
			_sdkLib.releaseCallbackHandlers()
			MessageBox.Show("Initialize failure.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		ListBoxSIPLog.Items.Add("Initialized.")
		_SIPInited = True

		loadDevices()

		' For make P2P call(PC 2 PC without SIP server), we just pass the user domain, 
		' SIP proxy server and outbound Proxy server, STUN server as empty

		Dim sipDomain As String = String.Empty
		Dim SIPServer As String = String.Empty
		Dim outboundServer As String = String.Empty
		Dim displayName As String = String.Empty
		Dim authName As String = String.Empty

		Dim SIPServerPort As Integer = 0
		Dim outboundServerPort As Integer = 0



		' Set the SIP user information
		rt = _sdkLib.setUser(userName, displayName, authName, password,
							sipDomain, SIPServer, SIPServerPort, stunServer, 0, outboundServer,
							outboundServerPort)
		If rt <> 0 Then
			_sdkLib.unInitialize()
			_sdkLib.releaseCallbackHandlers()

			_SIPInited = False
			ListBoxSIPLog.Items.Clear()

			MessageBox.Show("setUser failure.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		ListBoxSIPLog.Items.Add("Succeeded set user information.")


		SetSRTPType()

		Dim licenseKey As String = "PORTSIP_TEST_LICENSE"
		rt = _sdkLib.setLicenseKey(licenseKey)
		If rt = PortSIP_Errors.ECoreTrialVersionLicenseKey Then
			MessageBox.Show("This sample was built base on evaluation PortSIP VoIP SDK, which allows only three minutes conversation. The conversation will be cut off automatically after three minutes, then you can't hearing anything. Feel free contact us at: sales@portsip.com to purchase the official version.")
		End If

		If rt = PortSIP_Errors.ECoreWrongLicenseKey Then
			MessageBox.Show("The wrong license key was detected, please check with sales@portsip.com or support@portsip.com")
		End If


		SetVideoResolution()
		SetVideoQuality()

		UpdateAudioCodecs()
		UpdateVideoCodecs()

		InitSettings()
		updatePrackSetting()

		ListBoxSIPLog.Items.Add("Ready.")
	End Sub

	Private Sub ComboBoxSRTP_SelectedIndexChanged(sender As Object, e As EventArgs)
		SetSRTPType()
	End Sub

	Private Sub Button3_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "1"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 1, 160, True)
		End If
	End Sub

	Private Sub Button4_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "2"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 2, 160, True)
		End If
	End Sub

	Private Sub Button5_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "3"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 3, 160, True)
		End If
	End Sub

	Private Sub Button8_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "4"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 4, 160, True)
		End If
	End Sub

	Private Sub Button7_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "5"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 5, 160, True)
		End If
	End Sub

	Private Sub Button6_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "6"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 6, 160, True)
		End If
	End Sub

	Private Sub Button11_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "7"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 7, 160, True)
		End If
	End Sub

	Private Sub Button10_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "8"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 8, 160, True)
		End If
	End Sub

	Private Sub Button9_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "9"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 9, 160, True)
		End If
	End Sub

	Private Sub Button14_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "*"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 10, 160, True)
		End If
	End Sub

	Private Sub Button13_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "0"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 0, 160, True)
		End If
	End Sub

	Private Sub Button12_Click(sender As Object, e As EventArgs)
		TextBoxPhoneNumber.Text = TextBoxPhoneNumber.Text & "#"
		If _SIPInited = True AndAlso _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.sendDtmf(_CallSessions(_CurrentlyLine).getSessionId(), DTMF_METHOD.DTMF_RFC2833, 11, 160, True)
		End If
	End Sub

	Private Sub ComboBoxLines_SelectedIndexChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			ComboBoxLines.SelectedIndex = 0
			Return
		End If

		If _CurrentlyLine = (ComboBoxLines.SelectedIndex + LINE_BASE) Then
			Return
		End If

		If CheckBoxConf.Checked = True Then
			_CurrentlyLine = ComboBoxLines.SelectedIndex + LINE_BASE
			Return
		End If

		' To switch the line, must hold currently line first
		If _CallSessions(_CurrentlyLine).getSessionState() = True AndAlso _CallSessions(_CurrentlyLine).getHoldState() = False Then
			_sdkLib.hold(_CallSessions(_CurrentlyLine).getSessionId())
			_sdkLib.setRemoteVideoWindow(_CallSessions(_CurrentlyLine).getSessionId(), IntPtr.Zero)
			_CallSessions(_CurrentlyLine).setHoldState(True)

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Hold"
			ListBoxSIPLog.Items.Add(Text)
		End If

		_CurrentlyLine = ComboBoxLines.SelectedIndex + LINE_BASE

		' If target line was in hold state, then un-hold it
		If _CallSessions(_CurrentlyLine).getSessionState() = True AndAlso _CallSessions(_CurrentlyLine).getHoldState() = True Then
			_sdkLib.unHold(_CallSessions(_CurrentlyLine).getSessionId())
			_sdkLib.setRemoteVideoWindow(_CallSessions(_CurrentlyLine).getSessionId(), remoteVideoPanel.Handle)
			_CallSessions(_CurrentlyLine).setHoldState(False)

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": UnHold - call established"
			ListBoxSIPLog.Items.Add(Text)
		End If
	End Sub

	Private Sub ButtonDial_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If
		If TextBoxPhoneNumber.Text.Length <= 0 Then
			MessageBox.Show("The phone number is empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = True OrElse _CallSessions(_CurrentlyLine).getRecvCallState() = True Then
			MessageBox.Show("Current line is busy now, please switch a line.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If


		Dim callTo As String = TextBoxPhoneNumber.Text
		If callTo.IndexOf("sip:") = -1 OrElse callTo.IndexOf("@") = -1 Then
			MessageBox.Show("The phone number must likes: sip:123456@192.168.1.111.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		UpdateAudioCodecs()
		UpdateVideoCodecs()

		' Ensure the we have been added one audio codec at least
		If _sdkLib.isAudioCodecEmpty() = True Then
			InitDefaultAudioCodecs()
		End If

		'  Usually for 3PCC need to make call without SDP
		Dim hasSdp As [Boolean] = True
		If CheckBoxSDP.Checked = True Then
			hasSdp = False
		End If

		_sdkLib.setAudioDeviceId(ComboBoxMicrophones.SelectedIndex, ComboBoxSpeakers.SelectedIndex)

		Dim sessionId As Integer = _sdkLib.[call](callTo, hasSdp, checkBoxMakeVideoCall.Checked)
		If sessionId <= 0 Then
			ListBoxSIPLog.Items.Add("Failed to call.")
			Return
		End If

		_sdkLib.setRemoteVideoWindow(sessionId, remoteVideoPanel.Handle)

		_CallSessions(_CurrentlyLine).setSessionId(sessionId)
		_CallSessions(_CurrentlyLine).setSessionState(True)

		Dim Text As String = "Line " & _CurrentlyLine.ToString()
		Text = Text & ": Calling..."
		ListBoxSIPLog.Items.Add(Text)
	End Sub

	Private Sub ButtonHangUp_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getRecvCallState() = True Then
			_sdkLib.rejectCall(_CallSessions(_CurrentlyLine).getSessionId(), 486)
			_CallSessions(_CurrentlyLine).reset()

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Rejected call"
			ListBoxSIPLog.Items.Add(Text)

			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = True Then
			_sdkLib.hangUp(_CallSessions(_CurrentlyLine).getSessionId())
			_CallSessions(_CurrentlyLine).reset()

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Hang up"
			ListBoxSIPLog.Items.Add(Text)
		End If
	End Sub

	Private Sub ButtonAnswer_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getRecvCallState() = False Then
			MessageBox.Show("No incoming call on current line, please switch a line.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		_CallSessions(_CurrentlyLine).setRecvCallState(False)
		_CallSessions(_CurrentlyLine).setSessionState(True)


		_sdkLib.setRemoteVideoWindow(_CallSessions(_CurrentlyLine).getSessionId(), remoteVideoPanel.Handle)

		Dim rt As Integer = _sdkLib.answerCall(_CallSessions(_CurrentlyLine).getSessionId(), checkBoxAnswerVideo.Checked)
		If rt = 0 Then
			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Call established"
			ListBoxSIPLog.Items.Add(Text)

			joinConference(_CurrentlyLine)
		Else
			_CallSessions(_CurrentlyLine).reset()

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Failed to answer call."
			ListBoxSIPLog.Items.Add(Text)
		End If

	End Sub

	Private Sub ButtonReject_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getRecvCallState() = True Then
			_sdkLib.rejectCall(_CallSessions(_CurrentlyLine).getSessionId(), 486)
			_CallSessions(_CurrentlyLine).reset()

			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Rejected call"
			ListBoxSIPLog.Items.Add(Text)

			Return
		End If
	End Sub

	Private Sub ButtonHold_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False OrElse _CallSessions(_CurrentlyLine).getHoldState() = True Then
			Return
		End If

		Dim Text As String
		Dim rt As Integer = _sdkLib.hold(_CallSessions(_CurrentlyLine).getSessionId())
		If rt <> 0 Then
			Text = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": hold failed."
			ListBoxSIPLog.Items.Add(Text)

			Return
		End If

		_CallSessions(_CurrentlyLine).setHoldState(True)

		Text = "Line " & _CurrentlyLine.ToString()
		Text = Text & ": hold"
		ListBoxSIPLog.Items.Add(Text)
	End Sub

	Private Sub Button16_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False OrElse _CallSessions(_CurrentlyLine).getHoldState() = False Then
			Return
		End If

		Dim Text As String
		Dim rt As Integer = _sdkLib.unHold(_CallSessions(_CurrentlyLine).getSessionId())
		If rt <> 0 Then
			_CallSessions(_CurrentlyLine).setHoldState(False)

			Text = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Un-Hold Failure."
			ListBoxSIPLog.Items.Add(Text)

			Return
		End If


		_CallSessions(_CurrentlyLine).setHoldState(False)

		Text = "Line " & _CurrentlyLine.ToString()
		Text = Text & ": Un-Hold"
		ListBoxSIPLog.Items.Add(Text)
	End Sub

	Private Sub ButtonTransfer_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False Then
			MessageBox.Show("Need to make the call established first", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim TransferDlg As New TransferCallForm()
		If TransferDlg.ShowDialog() <> DialogResult.OK Then
			Return
		End If

		Dim referTo As String = TransferDlg.GetTransferNumber()

		If referTo.Length <= 0 Then
			MessageBox.Show("The transfer number is empty", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim rt As Integer = _sdkLib.refer(_CallSessions(_CurrentlyLine).getSessionId(), referTo)
		If rt <> 0 Then
			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Transfer failed"
			ListBoxSIPLog.Items.Add(Text)
		Else
			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Transferring"
			ListBoxSIPLog.Items.Add(Text)
		End If
	End Sub

	Private Sub button24_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False Then
			MessageBox.Show("Need to make the call established first", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim TransferDlg As New TransferCallForm()
		If TransferDlg.ShowDialog() <> DialogResult.OK Then
			Return
		End If

		Dim referTo As String = TransferDlg.GetTransferNumber()

		If referTo.Length <= 0 Then
			MessageBox.Show("The transfer number is empty", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim replaceLine As Integer = TransferDlg.GetReplaceLineNum()
		If replaceLine <= 0 OrElse replaceLine >= MAX_LINES Then
			MessageBox.Show("The replace line out of range", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		If _CallSessions(replaceLine).getSessionState() = False Then
			MessageBox.Show("The replace line does not established yet", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim rt As Integer = _sdkLib.attendedRefer(_CallSessions(_CurrentlyLine).getSessionId(), _CallSessions(replaceLine).getSessionId(), referTo)
		If rt <> 0 Then
			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Transfer failure"
			ListBoxSIPLog.Items.Add(Text)
		Else
			Dim Text As String = "Line " & _CurrentlyLine.ToString()
			Text = Text & ": Transferring"
			ListBoxSIPLog.Items.Add(Text)
		End If
	End Sub

	Private Sub CheckBoxConf_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			CheckBoxConf.Checked = False
			Return
		End If


		Dim width As Int32 = 352
		Dim height As Int32 = 288

		Select Case ComboBoxVideoResolution.SelectedIndex
			Case 0
				width = 176
				height = 144
				Exit Select
			Case 1
				width = 352
				height = 288
				Exit Select
			Case 2
				width = 640
				height = 480
				Exit Select
			Case 3
				width = 800
				height = 600
				Exit Select
			Case 4
				width = 1024
				height = 768
				Exit Select
			Case 5
				width = 1280
				height = 720
				Exit Select
			Case 6
				width = 320
				height = 240
				Exit Select
		End Select


		If CheckBoxConf.Checked = True Then
			Dim rt As Integer = _sdkLib.createVideoConference(remoteVideoPanel.Handle, width, height, 0)
			If rt = 0 Then
				ListBoxSIPLog.Items.Add("Make conference succeeded")
				For i As Integer = LINE_BASE To MAX_LINES - 1
					If _CallSessions(i).getSessionState() = True Then
						joinConference(i)
					End If
				Next
			Else
				ListBoxSIPLog.Items.Add("Failed to create conference.")
				CheckBoxConf.Checked = False
			End If
		Else
			' Stop conference

			' Before stop the conference, MUST place all lines to hold state

			Dim sessionIds As Integer() = New Integer(8) {0, 0, 0, 0, 0, 0,
			 0, 0, 0}
			For i As Integer = LINE_BASE To MAX_LINES - 1
				If _CallSessions(i).getSessionState() = True AndAlso _CallSessions(i).getHoldState() = False AndAlso _CurrentlyLine <> i Then

					sessionIds(i) = _CallSessions(i).getSessionId()

					' Hold it 
					_sdkLib.hold(sessionIds(i))
					_CallSessions(i).setHoldState(True)
				End If
			Next

			_sdkLib.destroyConference()

			If _CallSessions(_CurrentlyLine).getSessionState() = True AndAlso _CallSessions(_CurrentlyLine).getHoldState() = False Then
				_sdkLib.setRemoteVideoWindow(_CallSessions(_CurrentlyLine).getSessionId(), remoteVideoPanel.Handle)
			End If

			ListBoxSIPLog.Items.Add("Taken off Conference")
		End If
	End Sub

	Private Sub TrackBarSpeaker_ValueChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setSpeakerVolume(TrackBarSpeaker.Value)
	End Sub

	Private Sub TrackBarMicrophone_ValueChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setMicVolume(TrackBarMicrophone.Value)
	End Sub

	Private Sub CheckBoxMute_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If
		_sdkLib.muteMicrophone(CheckBoxMute.Checked)
	End Sub

	Private Sub ComboBoxMicrophones_SelectedIndexChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setAudioDeviceId(ComboBoxMicrophones.SelectedIndex, ComboBoxSpeakers.SelectedIndex)
	End Sub

	Private Sub ComboBoxSpeakers_SelectedIndexChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setAudioDeviceId(ComboBoxMicrophones.SelectedIndex, ComboBoxSpeakers.SelectedIndex)
	End Sub

	Private Sub ComboBoxCameras_SelectedIndexChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If
		_sdkLib.setVideoDeviceId(ComboBoxCameras.SelectedIndex)
	End Sub

	Private Sub ComboBoxVideoResolution_SelectedIndexChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		SetVideoResolution()
	End Sub

	Private Sub TrackBarVideoQuality_ValueChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		SetVideoQuality()
	End Sub

	Private Sub ButtonCameraOptions_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		Dim uniqueId As New StringBuilder()
		uniqueId.Length = 256
		Dim deviceName As New StringBuilder()
		deviceName.Length = 256

		Dim rt As Integer = _sdkLib.getVideoCaptureDeviceName(ComboBoxCameras.SelectedIndex, uniqueId, 256, deviceName, 256)
		If rt <> 0 Then
			MessageBox.Show("Get camera name failed.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		rt = _sdkLib.showVideoCaptureSettingsDialogBox(uniqueId.ToString(), uniqueId.Length, "Camera settings", Handle, 200, 200)
		If rt <> 0 Then
			MessageBox.Show("Failed to show the camera settings dialog.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

	End Sub

	Private Sub ButtonLocalVideo_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If ButtonLocalVideo.Text = "Local Video" Then
			_sdkLib.displayLocalVideo(True, True, localVideoPanel.Handle)
			ButtonLocalVideo.Text = "Stop Local"
		Else
			_sdkLib.displayLocalVideo(False, False, IntPtr.Zero)
			localVideoPanel.Refresh()
			ButtonLocalVideo.Text = "Local Video"
		End If
	End Sub

	Private Sub ButtonSendVideo_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False And _CallSessions(_CurrentlyLine).getRecvCallState() = False Then
			Return
		End If

		Dim rt As Integer = _sdkLib.sendVideo(_CallSessions(_CurrentlyLine).getSessionId(), True)
		If rt <> 0 Then
			MessageBox.Show("Start video sending failed.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
		End If
	End Sub

	Private Sub Button15_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		If _CallSessions(_CurrentlyLine).getSessionState() = False Then
			Return
		End If

		_sdkLib.sendVideo(_CallSessions(_CurrentlyLine).getSessionId(), False)
	End Sub


	Private Sub checkBoxPRACK_CheckedChanged(sender As Object, e As EventArgs)
		updatePrackSetting()
	End Sub

	Private Sub CheckBoxDND_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.setDoNotDisturb(CheckBoxDND.Checked)
	End Sub

	Private Sub checkBoxPCMU_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxPCMA_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxG729_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxILBC_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxGSM_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxAMR_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub


	Private Sub CheckBoxG722_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub CheckBoxG7221_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub CheckBoxAMRwb_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub CheckBoxSpeexWB_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub CheckBoxSpeex_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub

	Private Sub checkBoxOpus_CheckedChanged(sender As Object, e As EventArgs)
		UpdateAudioCodecs()
	End Sub


	Private Sub checkBoxH263_CheckedChanged(sender As Object, e As EventArgs)
		UpdateVideoCodecs()
	End Sub

	Private Sub checkBoxH2631998_CheckedChanged(sender As Object, e As EventArgs)
		UpdateVideoCodecs()
	End Sub

	Private Sub checkBoxH264_CheckedChanged(sender As Object, e As EventArgs)
		UpdateVideoCodecs()
	End Sub

	Private Sub checkBoxVP8_CheckedChanged(sender As Object, e As EventArgs)
		UpdateVideoCodecs()
	End Sub


	Private Sub checkBoxAEC_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.enableAEC(checkBoxAEC.Checked)


	End Sub

	Private Sub checkBoxVAD_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.enableVAD(checkBoxVAD.Checked)
	End Sub

	Private Sub checkBoxCNG_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.enableCNG(checkBoxAEC.Checked)
	End Sub

	Private Sub checkBoxAGC_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.enableAGC(checkBoxAGC.Checked)
	End Sub

	Private Sub checkBoxANS_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.enableANS(checkBoxANS.Checked)

	End Sub

	Private Sub Button17_Click(sender As Object, e As EventArgs)
		If FolderBrowserDialog1.ShowDialog() = DialogResult.OK Then
			TextBoxRecordFilePath.Text = FolderBrowserDialog1.SelectedPath
		End If
	End Sub

	Private Sub checkBox1_CheckedChanged(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			MessageBox.Show("Please initialize the SDK first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		' Callback the audio stream from each line
		Dim rt As Integer = _sdkLib.enableAudioStreamCallback(0, _CallSessions(_CurrentlyLine).getSessionId(), True, DIRECTION_MODE.DIRECTION_RECV)
		If rt <> 0 Then
			MessageBox.Show("Enable the audio stream callback failure.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			checkBox1.Checked = False
		End If
	End Sub

	Private Sub Button20_Click(sender As Object, e As EventArgs)
		If OpenFileDialog1.ShowDialog() = DialogResult.OK Then
			TextBoxPlayFile.Text = OpenFileDialog1.FileName
		End If
	End Sub

	Private Sub Button21_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			MessageBox.Show("Please initialize the SDK first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		If TextBoxPlayFile.Text.Length <= 0 Then
			MessageBox.Show("The play file is empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim playFile As String = TextBoxPlayFile.Text
		_sdkLib.startPlayingFileToRemote(_CallSessions(_CurrentlyLine).getSessionId(), playFile, False, 1)

	End Sub

	Private Sub Button23_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.stopPlayingFileToRemote(_CallSessions(_CurrentlyLine).getSessionId())
	End Sub

	Private Sub button27_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			MessageBox.Show("Please initialize the SDK first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

			Return
		End If

		Dim forwardTo As String = textBoxForwardTo.Text
		If forwardTo.IndexOf("sip:") = -1 OrElse forwardTo.IndexOf("@") = -1 Then
			MessageBox.Show("The forward address must likes sip:xxxx@sip.portsip.com.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

			Return
		End If

		If checkBoxForwardCallForBusy.Checked = True Then
			_sdkLib.enableCallForward(True, forwardTo)
		Else
			_sdkLib.enableCallForward(False, forwardTo)
		End If

		MessageBox.Show("The call forward is enabled.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

	End Sub

	Private Sub button28_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			MessageBox.Show("Please initialize the SDK first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

			Return
		End If

		_sdkLib.disableCallForward()

		MessageBox.Show("The call forward is disabled.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
	End Sub

	Private Sub linkLabel1_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs)
		System.Diagnostics.Process.Start("http://portsip.com/portsip-pbx")
	End Sub

	Private Sub linkLabel2_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs)
		System.Diagnostics.Process.Start("mailto:sales@portsip.com")
	End Sub

	Private Sub Button22_Click(sender As Object, e As EventArgs)
		ListBoxSIPLog.Items.Clear()
	End Sub

	Private Sub Button19_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			MessageBox.Show("Please initialize the SDK first.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		If TextBoxRecordFilePath.Text.Length <= 0 OrElse TextBoxRecordFileName.Text.Length <= 0 Then
			MessageBox.Show("The file path or file name is empty.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		Dim filePath As String = TextBoxRecordFilePath.Text
		Dim fileName As String = TextBoxRecordFileName.Text

		Dim recordFileFormat As FILE_FORMAT = FILE_FORMAT.FILEFORMAT_WAVE

		'  Start recording
		Dim rt As Integer = _sdkLib.startRecord(_CallSessions(_CurrentlyLine).getSessionId(), filePath, fileName, True, 1, recordFileFormat, RECORD_MODE.RECORD_BOTH, RECORD_MODE.RECORD_RECV)
		If rt <> 0 Then
			MessageBox.Show("Failed to start record conversation.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
			Return
		End If

		MessageBox.Show("Started record conversation.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
	End Sub

	Private Sub Button18_Click(sender As Object, e As EventArgs)
		If _SIPInited = False Then
			Return
		End If

		_sdkLib.stopRecord(_CallSessions(_CurrentlyLine).getSessionId())

		MessageBox.Show("Stop record conversation.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
	End Sub


	'''///////////////////////////////////////////////////////////////////////////////////////////////////////////
	'''///////////////////////////////////////////////////////////////////////////////////////////////////////////

	''' <summary>
	'''  With below all onXXX functions, you MUST use the Invoke/BeginInvoke method if you want
	'''  modify any control on the Forms.
	'''  More details please visit: http://msdn.microsoft.com/en-us/library/ms171728.aspx
	'''  The Invoke method is recommended.
	'''  
	'''  if you don't like Invoke/BeginInvoke method, then  you can add this line to Form_Load:
	'''  Control.CheckForIllegalCrossThreadCalls = false;
	'''  This requires .NET 2.0 or higher
	''' 
	''' </summary>
	''' 
	Public Function onRegisterSuccess(statusText As [String], statusCode As Int32, sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onRegisterSuccess
		' For P2P mode, do not need to process register events.
		' Use the Invoke method to modify the control.
		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add("Registration succeeded")))

		Return 0
	End Function


	Public Function onRegisterFailure(ByVal statusText As [String], ByVal statusCode As Int32, sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onRegisterFailure
		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add("Registration failure")))

		Return 0
	End Function

	Public Function onInviteIncoming(ByVal sessionId As Int32, ByVal callerDisplayName As [String], ByVal caller As [String], ByVal calleeDisplayName As [String],
	 ByVal callee As [String], ByVal audioCodecNames As [String], ByVal videoCodecNames As [String],
	 ByVal existsAudio As [Boolean], ByVal existsVideo As [Boolean], ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteIncoming
		Dim index As Integer = -1
		For i As Integer = LINE_BASE To MAX_LINES - 1
			If _CallSessions(i).getSessionState() = False AndAlso _CallSessions(i).getRecvCallState() = False Then
				index = i
				_CallSessions(i).setRecvCallState(True)
				Exit For
			End If
		Next

		If index = -1 Then
			_sdkLib.rejectCall(sessionId, 486)
			Return 0
		End If

		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsVideo Then
		End If
		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsAudio Then
		End If

		_CallSessions(index).setSessionId(sessionId)
		Dim Text As String = String.Empty

		Dim needIgnoreAutoAnswer As Boolean = False
		Dim j As Integer = 0

		For j = LINE_BASE To MAX_LINES - 1
			If _CallSessions(j).getSessionState() = True Then
				needIgnoreAutoAnswer = True
				Exit For
			End If
		Next

		Dim AA As [Boolean] = False
		Dim answerVideoCall As [Boolean] = False
		ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
			  AA = CheckBoxAA.Checked))

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
		answerVideoCall = checkBoxAnswerVideo.Checked))



		If needIgnoreAutoAnswer = False AndAlso AA = True Then
			_CallSessions(index).setRecvCallState(False)
			_CallSessions(index).setSessionState(True)


			ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
			 _sdkLib.setRemoteVideoWindow(_CallSessions(index).getSessionId(), remoteVideoPanel.Handle)))



			_sdkLib.answerCall(_CallSessions(index).getSessionId(), answerVideoCall)

			Text = "Line " & index.ToString()
			Text = Text & ": Answered call by Auto answer"
			ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

			Return 0
		End If

		Text = "Line " & index.ToString()
		Text = Text & ": Call incoming from "
		Text = Text & callerDisplayName
		Text = Text & "<"
		Text = Text & caller
		Text = Text & ">"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		'  You should write your own code to play the wav file here for alert the incoming call(incoming tone);

		Return 0

	End Function

	Public Function onInviteTrying(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onInviteTrying
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call is trying..."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onInviteSessionProgress(ByVal sessionId As Int32, ByVal audioCodecNames As [String], ByVal videoCodecNames As [String], ByVal existsEarlyMedia As [Boolean],
	 ByVal existsAudio As [Boolean], ByVal existsVideo As [Boolean], ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteSessionProgress
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsVideo Then
		End If
		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsAudio Then
		End If


		_CallSessions(i).setSessionState(True)

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call session progress."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		_CallSessions(i).setEarlyMeida(existsEarlyMedia)

		Return 0
	End Function

	Public Function onInviteRinging(ByVal sessionId As Int32,
									ByVal statusText As [String], ByVal statusCode As Int32, ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteRinging
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		' No early media, you must play the local WAVE  file for ringing tone
		If _CallSessions(i).hasEarlyMedia() = False Then
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Ringing..."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))


		Return 0
	End Function


	Public Function onInviteAnswered(ByVal sessionId As Int32, ByVal callerDisplayName As [String], ByVal caller As [String], ByVal calleeDisplayName As [String],
	 ByVal callee As [String], ByVal audioCodecNames As [String], ByVal videoCodecNames As [String], ByVal existsAudio As [Boolean], ByVal existsVideo As [Boolean], ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteAnswered
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsVideo Then
		End If
		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsAudio Then
		End If

		_CallSessions(i).setSessionState(True)

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call established"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
			ListBoxSIPLog.Items.Add(Text)))

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
		joinConference(i)))

		' If this is the refer call then need set it to normal
		If _CallSessions(i).isReferCall() Then
			_CallSessions(i).setReferCall(False, 0)
		End If

		Return 0
	End Function


	Public Function onInviteFailure(ByVal sessionId As Int32,
									ByVal callerDisplayName As [String],
									ByVal caller As [String],
									ByVal calleeDisplayName As [String],
									ByVal callee As [String],
									ByVal reason As [String],
									ByVal code As Int32,
									ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteFailure
		Dim index As Integer = findSession(sessionId)
		If index = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & index.ToString()
		Text += ": call failure, "
		Text += reason
		Text += ", "
		Text += code.ToString()

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))


		If _CallSessions(index).isReferCall() Then
			' Take off the origin call from HOLD if the refer call is failure
			Dim originIndex As Integer = -1
			For i As Integer = LINE_BASE To MAX_LINES - 1
				' Looking for the origin call
				If _CallSessions(i).getSessionId() = _CallSessions(index).getOriginCallSessionId() Then
					originIndex = i
					Exit For
				End If
			Next

			If originIndex <> -1 Then
				_sdkLib.unHold(_CallSessions(index).getOriginCallSessionId())
				_CallSessions(originIndex).setHoldState(False)

				' Switch the currently line to origin call line
				_CurrentlyLine = originIndex
				ComboBoxLines.SelectedIndex = _CurrentlyLine - 1
				Text = "Current line is set to: "
				Text += _CurrentlyLine.ToString()

				ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))
			End If
		End If

		_CallSessions(index).reset()

		Return 0
	End Function


	Public Function onInviteUpdated(ByVal sessionId As Int32, ByVal audioCodecNames As [String], ByVal videoCodecNames As [String], ByVal existsAudio As [Boolean],
	 ByVal existsVideo As [Boolean], ByVal existsScreen As [Boolean], ByVal sipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onInviteUpdated
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If


		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsVideo Then
		End If
		' If more than one codecs using, then they are separated with "#",
		' for example: "g.729#GSM#AMR", "H264#H263", you have to parse them by yourself.
		If existsAudio Then
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call is updated"
		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function

	Public Function onInviteConnected(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onInviteConnected
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call is connected"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onInviteBeginingForward(ByVal forwardTo As [String]) As Int32 Implements SIPCallbackEvents.onInviteBeginingForward
		Dim Text As String = "Call has been forwarded to: "
		Text = Text & forwardTo

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onInviteClosed(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onInviteClosed
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		_CallSessions(i).reset()

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Call closed"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function

	Public Function onDialogStateUpdated(BLFMonitoredUri As [String], BLFDialogState As [String], BLFDialogId As [String], BLFDialogDirection As [String]) As Int32 Implements SIPCallbackEvents.onDialogStateUpdated


		Dim Text As String = "The user "
		Text += BLFMonitoredUri
		Text += " dialog state is updated: "
		Text += BLFDialogState
		Text += ", dialog id: "
		Text += BLFDialogId
		Text += ", direction: "
		Text += BLFDialogDirection

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onRemoteHold(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onRemoteHold
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Placed on hold by remote party"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onRemoteUnHold(ByVal sessionId As Int32, ByVal audioCodecNames As [String], ByVal videoCodecNames As [String], ByVal existsAudio As [Boolean],
	 ByVal existsVideo As [Boolean]) As Int32 Implements SIPCallbackEvents.onRemoteUnHold
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Take off hold by remote party"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onReceivedRefer(ByVal sessionId As Int32, ByVal referId As Int32, ByVal [to] As [String], ByVal from As [String],
	 ByVal referSipMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onReceivedRefer
		Dim index As Integer = findSession(sessionId)
		If index = -1 Then
			_sdkLib.rejectRefer(referId)
			Return 0
		End If

		Dim Text As String = "Received REFER on line "
		Text += index.ToString()
		Text += ", refer to "
		Text += [to]

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() _
			  ListBoxSIPLog.Items.Add(Text)))


		' Accept the REFER automatically
		Dim referSessionId As Integer = _sdkLib.acceptRefer(referId, referSipMessage.ToString())
		If referSessionId <= 0 Then
			Text = "Failed to accept REFER on line"
			ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))
		Else
			_sdkLib.hangUp(_CallSessions(index).getSessionId())
			_CallSessions(index).reset()

			_CallSessions(index).setSessionId(referSessionId)
			_CallSessions(index).setSessionState(True)


			Text = "Accepted the REFER."
			ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))
		End If

		Return 0
	End Function


	Public Function onReferAccepted(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onReferAccepted
		Dim index As Integer = findSession(sessionId)
		If index = -1 Then
			Return 0
		End If

		Dim Text As String = "Line "
		Text += index.ToString()
		Text += ", the REFER was accepted"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function



	Public Function onReferRejected(ByVal sessionId As Int32, ByVal reason As [String], ByVal code As Int32) As Int32 Implements SIPCallbackEvents.onReferRejected
		Dim index As Integer = findSession(sessionId)
		If index = -1 Then
			Return 0
		End If

		Dim Text As String = "Line "
		Text += index.ToString()
		Text += ", the REFER was rejected"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onTransferTrying(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onTransferTrying
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Transfer Trying"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))


		Return 0
	End Function

	Public Function onTransferRinging(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onTransferRinging
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Transfer Ringing"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function



	Public Function onACTVTransferSuccess(ByVal sessionId As Int32) As Int32 Implements SIPCallbackEvents.onACTVTransferSuccess
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		_sdkLib.hangUp(_CallSessions(i).getSessionId())
		_CallSessions(i).reset()

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Transfer succeeded, close the call."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onACTVTransferFailure(ByVal sessionId As Int32, ByVal reason As [String], ByVal code As Int32) As Int32 Implements SIPCallbackEvents.onACTVTransferFailure
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Line " & i.ToString()
		Text = Text & ": Transfer failure"

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		'  reason is error reason
		'  code is error code

		Return 0
	End Function

	Public Function onReceivedSignaling(ByVal sessionId As Int32, ByVal signaling As StringBuilder) As Int32 Implements SIPCallbackEvents.onReceivedSignaling
		' This event will be fired when the SDK received a SIP message
		' you can use signaling to access the SIP message.

		Return 0
	End Function


	Public Function onSendingSignaling(ByVal sessionId As Int32, ByVal signaling As StringBuilder) As Int32 Implements SIPCallbackEvents.onSendingSignaling
		' This event will be fired when the SDK sent a SIP message
		' you can use signaling to access the SIP message.

		Return 0
	End Function




	Public Function onWaitingVoiceMessage(ByVal messageAccount As [String], ByVal urgentNewMessageCount As Int32, ByVal urgentOldMessageCount As Int32, ByVal newMessageCount As Int32,
	 ByVal oldMessageCount As Int32) As Int32 Implements SIPCallbackEvents.onWaitingVoiceMessage
		Dim Text As String = messageAccount
		Text += " has voice message."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		' You can use these parameters to check the voice message count

		'  urgentNewMessageCount;
		'  urgentOldMessageCount;
		'  newMessageCount;
		'  oldMessageCount;

		Return 0
	End Function


	Public Function onWaitingFaxMessage(ByVal messageAccount As [String], ByVal urgentNewMessageCount As Int32, ByVal urgentOldMessageCount As Int32, ByVal newMessageCount As Int32,
	 ByVal oldMessageCount As Int32) As Int32 Implements SIPCallbackEvents.onWaitingFaxMessage
		Dim Text As String = messageAccount
		Text += " has FAX message."

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		' You can use these parameters to check the FAX message count

		'  urgentNewMessageCount;
		'  urgentOldMessageCount;
		'  newMessageCount;
		'  oldMessageCount;

		Return 0
	End Function


	Public Function onRecvDtmfTone(ByVal sessionId As Int32, ByVal tone As Int32) As Int32 Implements SIPCallbackEvents.onRecvDtmfTone
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If


		Dim DTMFTone As String = tone.ToString()
		Select Case tone
			Case 10
				DTMFTone = "*"
				Exit Select

			Case 11
				DTMFTone = "#"
				Exit Select

			Case 12
				DTMFTone = "A"
				Exit Select

			Case 13
				DTMFTone = "B"
				Exit Select

			Case 14
				DTMFTone = "C"
				Exit Select

			Case 15
				DTMFTone = "D"
				Exit Select

			Case 16
				DTMFTone = "FLASH"
				Exit Select
		End Select


		Dim Text As String = "Received DTMF Tone: "
		Text += DTMFTone
		Text += " on line "
		Text += i.ToString()

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onPresenceRecvSubscribe(ByVal subscribeId As Int32, ByVal fromDisplayName As [String], ByVal from As [String], ByVal subject As [String]) As Int32 Implements SIPCallbackEvents.onPresenceRecvSubscribe


		Return 0
	End Function


	Public Function onPresenceOnline(ByVal fromDisplayName As [String], ByVal from As [String], ByVal stateText As [String]) As Int32 Implements SIPCallbackEvents.onPresenceOnline

		Return 0
	End Function

	Public Function onPresenceOffline(ByVal fromDisplayName As [String], ByVal from As [String]) As Int32 Implements SIPCallbackEvents.onPresenceOffline


		Return 0
	End Function


	Public Function onRecvOptions(ByVal optionsMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onRecvOptions
		'  string text = "Received an OPTIONS message: ";
		'  text += optionsMessage.ToString();
		'  MessageBox.Show(text, "Received an OPTIONS message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

		Return 0
	End Function

	Public Function onRecvInfo(ByVal infoMessage As StringBuilder) As Int32 Implements SIPCallbackEvents.onRecvInfo
		Dim text As String = "Received a INFO message: "
		text += infoMessage.ToString()

		MessageBox.Show(text, "Received a INFO message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

		Return 0
	End Function



	Public Function onRecvNotifyOfSubscription(subscribeId As Int32, notifyMsg As StringBuilder, contentData As Byte(), contentLenght As Int32) As Int32 Implements SIPCallbackEvents.onRecvNotifyOfSubscription

		Return 0
	End Function

	Public Function onSubscriptionFailure(subscribeId As Int32, statusCode As Int32) As Int32 Implements SIPCallbackEvents.onSubscriptionFailure
		Return 0
	End Function

	Public Function onSubscriptionTerminated(subscribeId As Int32) As Int32 Implements SIPCallbackEvents.onSubscriptionTerminated

		Return 0
	End Function


	Public Function onRecvMessage(ByVal sessionId As Int32, ByVal mimeType As [String], ByVal subMimeType As [String], ByVal messageData As Byte(),
	 ByVal messageDataLength As Int32) As Int32 Implements SIPCallbackEvents.onRecvMessage
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim text As String = "Received a MESSAGE message on line "
		text += i.ToString()

		If mimeType = "text" AndAlso subMimeType = "plain" Then
			Dim mesageText As String = GetString(messageData)
			' The messageData is binary data
		ElseIf mimeType = "application" AndAlso subMimeType = "vnd.3gpp.sms" Then
			' The messageData is binary data
		ElseIf mimeType = "application" AndAlso subMimeType = "vnd.3gpp2.sms" Then
		End If

		MessageBox.Show(text, "Received a MESSAGE message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

		Return 0
	End Function


	Public Function onRecvOutOfDialogMessage(ByVal fromDisplayName As [String], ByVal from As [String], ByVal toDisplayName As [String], ByVal [to] As [String],
	 ByVal mimeType As [String], ByVal subMimeType As [String], ByVal messageData As Byte(), ByVal messageDataLength As Int32) As Int32 Implements SIPCallbackEvents.onRecvOutOfDialogMessage
		Dim text As String = "Received a message(out of dialog) from "
		text += from

		If mimeType = "text" AndAlso subMimeType = "plain" Then
			Dim mesageText As String = GetString(messageData)
			' The messageData is binary data
		ElseIf mimeType = "application" AndAlso subMimeType = "vnd.3gpp.sms" Then
			' The messageData is binary data
		ElseIf mimeType = "application" AndAlso subMimeType = "vnd.3gpp2.sms" Then
		End If

		MessageBox.Show(text, "Received a out of dialog MESSAGE message", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

		Return 0
	End Function

	Public Function onSendMessageSuccess(ByVal sessionId As Int32, ByVal messageId As Int32) As Int32 Implements SIPCallbackEvents.onSendMessageSuccess
		Return 0
	End Function


	Public Function onSendMessageFailure(ByVal sessionId As Int32, ByVal messageId As Int32, ByVal reason As [String], ByVal code As Int32) As Int32 Implements SIPCallbackEvents.onSendMessageFailure

		Return 0
	End Function



	Public Function onSendOutOfDialogMessageSuccess(ByVal messageId As Int32, ByVal fromDisplayName As [String], ByVal from As [String], ByVal toDisplayName As [String],
	 ByVal [to] As [String]) As Int32 Implements SIPCallbackEvents.onSendOutOfDialogMessageSuccess


		Return 0
	End Function

	Public Function onSendOutOfDialogMessageFailure(ByVal messageId As Int32, ByVal fromDisplayName As [String], ByVal from As [String], ByVal toDisplayName As [String],
	 ByVal [to] As [String], ByVal reason As [String], ByVal code As Int32) As Int32 Implements SIPCallbackEvents.onSendOutOfDialogMessageFailure
		Return 0
	End Function


	Public Function onPlayFileFinished(ByVal sessionId As Int32, ByVal fileName As [String]) As Int32 Implements SIPCallbackEvents.onPlayFileFinished
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "Play file - "
		Text += fileName
		Text += " end on line: "
		Text += i.ToString()

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function

	Public Function onStatistics(ByVal sessionId As Int32, stat As [String]) As Int32 Implements SIPCallbackEvents.onStatistics
		Dim i As Integer = findSession(sessionId)
		If i = -1 Then
			Return 0
		End If

		Dim Text As String = "onStatisticson line: "
		Text += i.ToString()

		ListBoxSIPLog.Invoke(New MethodInvoker(Function() ListBoxSIPLog.Items.Add(Text)))

		Return 0
	End Function


	Public Function onRTPPacketCallback(callbackObject As IntPtr, sessionId As Int32, mediaType As Int32, direction As Int32, RTPPacket As Byte(), packetSize As Int32) As Int32 Implements SIPCallbackEvents.onRTPPacketCallback
		'
		'                !!! IMPORTANT !!!
		'
		'                Don't call any PortSIP SDK API functions in here directly. If you want to call the PortSIP API functions or 
		'                other code which will spend long time, you should post a message to main thread(main window) or other thread,
		'                let the thread to call SDK API functions or other code.
		'
		'            


		Return 0
	End Function

	Public Function onAudioRawCallback(ByVal callbackObject As IntPtr, ByVal sessionId As Int32, ByVal callbackType As Int32, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=4)> ByVal data As Byte(), ByVal dataLength As Int32, ByVal samplingFreqHz As Int32) As Int32 Implements SIPCallbackEvents.onAudioRawCallback

		'
		'                !!! IMPORTANT !!!
		'
		'                Don't call any PortSIP SDK API functions in here directly. If you want to call the PortSIP API functions or 
		'                other code which will spend long time, you should post a message to main thread(main window) or other thread,
		'                let the thread to call SDK API functions or other code.
		'
		'            


		' The data parameter is audio stream as PCM format, 16bit, Mono.
		' the dataLength parameter is audio steam data length.


		'
		' IMPORTANT: the data length is stored in dataLength parameter!!!
		'

		Dim type As DIRECTION_MODE = CType(callbackType, DIRECTION_MODE)

		If type = DIRECTION_MODE.DIRECTION_SEND Then
			' The callback data is from local record device of each session, use the sessionId to identifying the session.
		ElseIf type = DIRECTION_MODE.DIRECTION_RECV Then
			' The callback data is received from remote side of each session, use the sessionId to identifying the session.
		End If

		Return 0
	End Function


	Public Function onVideoRawCallback(ByVal callbackObject As IntPtr, ByVal sessionId As Int32, ByVal callbackType As Int32, ByVal width As Int32, ByVal height As Int32, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=6)> ByVal data As Byte(),
	 ByVal dataLength As Int32) As Int32 Implements SIPCallbackEvents.onVideoRawCallback
		'
		'                !!! IMPORTANT !!!
		'
		'                Don't call any PortSIP SDK API functions in here directly. If you want to call the PortSIP API functions or 
		'                other code which will spend long time, you should post a message to main thread(main window) or other thread,
		'                let the thread to call SDK API functions or other code.
		'
		'                The video data format is YUV420, YV12.
		'            


		'
		' IMPORTANT: the data length is stored in dataLength parameter!!!
		'

		Dim type As DIRECTION_MODE = CType(callbackType, DIRECTION_MODE)



		If type = DIRECTION_MODE.DIRECTION_SEND Then

		ElseIf type = DIRECTION_MODE.DIRECTION_RECV Then
		End If

		Return 0

	End Function
	Public Function onScreenRawCallback(ByVal callbackObject As IntPtr, ByVal sessionId As Int32, ByVal callbackType As Int32, ByVal width As Int32, ByVal height As Int32, <MarshalAs(UnmanagedType.LPArray, SizeParamIndex:=6)> ByVal data As Byte(),
	 ByVal dataLength As Int32) As Int32 Implements SIPCallbackEvents.onScreenRawCallback
		'
		'                !!! IMPORTANT !!!
		'
		'                Don't call any PortSIP SDK API functions in here directly. If you want to call the PortSIP API functions or 
		'                other code which will spend long time, you should post a message to main thread(main window) or other thread,
		'                let the thread to call SDK API functions or other code.
		'
		'                The video data format is YUV420, YV12.
		'            


		'
		' IMPORTANT: the data length is stored in dataLength parameter!!!
		'

		Dim type As DIRECTION_MODE = CType(callbackType, DIRECTION_MODE)



		If type = DIRECTION_MODE.DIRECTION_SEND Then

		ElseIf type = DIRECTION_MODE.DIRECTION_RECV Then
		End If

		Return 0

	End Function


End Class
